import augeas
from jadi import interface


class AugeasError(Exception):
    def __init__(self, aug):
        self.message = None
        self.data = {}
        aug.dump('/')
        for ep in aug.match('/augeas//error'):
            self.message = aug.get(ep + '/message')
            for p in aug.match(ep + '/*'):
                self.data[p.split('/')[-1]] = aug.get(p)

    def __str__(self):
        return self.message


class Augeas(augeas.Augeas):
    """
    A smarter and faster wrapper around :class:`augeas.Augeas`.augeas

    For faster startup, no modules and lenses are preloaded::

        aug = Augeas(modules=[{
            'name': 'Interfaces',  # module name
            'lens': 'Interfaces.lns',  # lens name
            'incl': [  # included files list
                self.path,
                self.path + '.d/*',
            ]
        }])

    Don't forget to call :func:`.load()` afterwards.
    """

    def __init__(self, modules=[], loadpath=None):
        augeas.Augeas.__init__(self, loadpath=loadpath, flags=augeas.Augeas.NO_MODL_AUTOLOAD | augeas.Augeas.NO_LOAD)
        for module in modules:
            path = '/augeas/load/%s' % module['name']
            self.set(path + '/lens', module['lens'])
            for index, incl in enumerate(module['incl']):
                self.set(path + '/incl[%i]' % (index + 1), incl)

    def __enc(self, v):
        if v:
            return v.encode('utf8')

    def match(self, path):
        return augeas.Augeas.match(self, self.__enc(path))

    def set(self, path, value):
        augeas.Augeas.set(self, self.__enc(path), self.__enc(value))

    def setd(self, path, value, default=None):
        """
        Sets `path` to `value`, or removes `path` if `value == default`
        """
        self.set(path, value)
        if value == default:
            self.remove(path)

    def raise_error(self):
        """
        Extracts error information from Augeas tree and raises :exc:`AugeasError`
        """
        raise AugeasError(self)

    def save(self):
        try:
            augeas.Augeas.save(self)
        except IOError:
            self.raise_error()

    def dump(self, path):
        """
        Dumps contents under `path` to stdout.
        """
        for sp in self.match(path + '/*'):
            print sp, '=', self.get(sp)
            self.dump(sp)



import multiprocessing
from jadi import component

from vvv.api.config import SystemConfig
from vvv.api.plugin import Plugin


@component(Plugin)
class NginxPlugin(Plugin):
    name = 'nginx'

    def add_config_defaults(self, config):
        if isinstance(config, SystemConfig):
            config.data.setdefault('nginx', {})
            config.data['nginx'].setdefault('config_root', '/etc/nginx')
            config.data['nginx'].setdefault('config_file', '/etc/nginx/nginx.conf')
            config.data['nginx'].setdefault('config_file_mime', '/etc/nginx/mime.conf')
            config.data['nginx'].setdefault('config_file_fastcgi', '/etc/nginx/fcgi.conf')
            config.data['nginx'].setdefault('config_file_proxy', '/etc/nginx/proxy.conf')
            config.data['nginx'].setdefault('config_vhost_root', '/etc/nginx/conf.d')
            config.data['nginx'].setdefault('config_custom_root', '/etc/nginx.custom.d')
            config.data['nginx'].setdefault('lib_path', '/var/lib/nginx')
            config.data['nginx'].setdefault('user', 'www-data')
            config.data['nginx'].setdefault('workers', multiprocessing.cpu_count())



import logging
import os
import subprocess
from jadi import component

from vvv.api.app import AppType
from vvv.api.aug import Augeas
from vvv.api.config import MainConfig, SystemConfig
from vvv.api.configurable import Configurable
from vvv.api.check import Check, CheckFailure
from vvv.api.restartable import Restartable

from .plugin import SupervisorImpl


@component(Check)
class AppsCheck(Check):
    def get_instances(self):
        for website in MainConfig.get(self.context).data['websites']:
            if website['enabled']:
                prefix = 'veb-app-%s-' % website['name']
                for app in website['apps']:
                    app_type = AppType.by_name(self.context, app['type'])
                    if not app_type:
                        logging.warn('Skipping unknown app type "%s"', app['type'])
                        continue
                    process_info = app_type.get_process(website, app)
                    if process_info:
                        full_name = prefix + app['name'] + process_info.get('suffix', '')
                        yield [{
                            'full_name': full_name,
                            'website': website,
                            'app': app,
                        }]

    def get_name(self, info):
        return 'app %s/%s is running' % (
            info['website']['name'],
            info['app']['name'],
        )

    def run(self, info):
        o = subprocess.check_output(
            ['supervisorctl', 'status', info['full_name']]
        )
        if 'RUNNING' not in o:
            raise CheckFailure(o)
        return True


@component(Check)
class SupervisorServiceCheck(Check):
    name = 'supervisor is running'

    def run(self):
        return subprocess.call(
            ['service', SupervisorImpl.any(self.context).service_name, 'status']
        ) == 0


@component(AppType)
class GenericAppType(AppType):
    name = 'generic'

    def get_access_type(self, website_config, app):
        return None

    def get_access_params(self, website_config, app):
        return {}

    def get_process(self, website_config, app):
        return {
            'command': app['params']['command'],
            'directory': app['params']['directory'] or website_config['root'],
            'environment': app['params']['environment'],
            'user': app['params']['user'],
        }


@component(Configurable)
class Supervisor(Configurable):
    name = 'supervisor'

    def configure(self):
        aug = Augeas(
            modules=[{
                'name': 'Supervisor',
                'lens': 'Supervisor.lns',
                'incl': [
                    SystemConfig.get(self.context).data['supervisor']['config_file'],
                ]
            }],
            loadpath=os.path.dirname(__file__),
        )
        aug_path = '/files' + SystemConfig.get(self.context).data['supervisor']['config_file']
        aug.load()

        for website in MainConfig.get(self.context).data['websites']:
            if website['enabled']:
                prefix = 'veb-app-%s-' % website['name']

                for path in aug.match(aug_path + '/*'):
                    if aug.get(path + '/#comment') == 'Autogenerated Ajenti V process':
                        aug.remove(path)
                    if prefix in path:
                        aug.remove(path)

                for app in website['apps']:
                    app_type = AppType.by_name(self.context, app['type'])
                    if not app_type:
                        logging.warn('Skipping unknown app type "%s"', app['type'])
                        continue

                    process_info = app_type.get_process(website, app)
                    if process_info:
                        full_name = prefix + app['name'] + process_info.get('suffix', '')
                        path = aug_path + '/program:%s' % full_name
                        aug.set(path + '/command', process_info['command'])
                        aug.set(path + '/directory', process_info['directory'] or website['root'])
                        if process_info['environment']:
                            aug.set(path + '/environment', process_info['environment'])
                        aug.set(path + '/user', process_info['user'])
                        aug.set(path + '/killasgroup', 'true')
                        aug.set(path + '/stopasgroup', 'true')
                        aug.set(path + '/stdout_logfile', '%s/%s/%s.stdout.log' % (
                            SystemConfig.get(self.context).data['log_dir'],
                            website['name'],
                            app['name'],
                        ))
                        aug.set(path + '/stderr_logfile', '%s/%s/%s.stderr.log' % (
                            SystemConfig.get(self.context).data['log_dir'],
                            website['name'],
                            app['name'],
                        ))

        aug.save()
        SupervisorRestartable.get(self.context).schedule_restart()


@component(Restartable)
class SupervisorRestartable(Restartable):
    name = 'supervisor'

    def do_restart(self):
        subprocess.check_call(['service', SupervisorImpl.any(self.context).service_name, 'restart'])



from jadi import interface


@interface
class Check(object):
    name = None

    def __init__(self, context):
        self.context = context

    def get_instances(self):
        return [[]]

    def get_name(self, *args):
        return self.name

    def run(self, *args):
        pass


class CheckFailure(Exception):
    pass



from jadi import interface


@interface
class AppType(object):
    name = None

    @classmethod
    def by_name(cls, context, name):
        for app in cls.classes():
            if app.name == name:
                return app(context)

    def __init__(self, context):
        self.context = context

    def get_access_type(self, website_config, app_config):
        raise NotImplementedError

    def get_access_params(self, website_config, app_config):
        raise NotImplementedError

    def get_process(self, website_config, app_config):
        return None
        


from jadi import component

from vvv.api.plugin import Plugin
from vvv.api.config import MainConfig, SystemConfig


@component(Plugin)
class CorePlugin(Plugin):
    name = 'core'

    def add_config_defaults(self, config):
        if isinstance(config, MainConfig):
            config.data.setdefault('websites', [])

            for ws in config.data['websites']:
                ws.setdefault('name', 'unnamed')
                ws.setdefault('enabled', True)

                ws.setdefault('domains', [])
                for domain in ws['domains']:
                    domain.setdefault('domain', 'example.com')

                ws.setdefault('apps', [])
                for app in ws['apps']:
                    app.setdefault('path', '/srv')
                    app.setdefault('name', 'unnamed')
                    app.setdefault('type', 'php-fpm')
                    app.setdefault('params', {})

                ws.setdefault('ports', [])
                for port in ws['ports']:
                    port.setdefault('host', None)
                    port.setdefault('port', 80)
                    port.setdefault('default', False)
                    port.setdefault('ssl', False)
                    port.setdefault('spdy', False)

                ws.setdefault('ssl_cert_path', None)
                ws.setdefault('ssl_key_path', None)
                ws.setdefault('root', '/srv')

                ws.setdefault('custom_conf', None)
                ws.setdefault('custom_conf_toplevel', None)
                ws.setdefault('maintenance_mode', False)

                ws.setdefault('locations', [])
                for location in ws['locations']:
                    location.setdefault('pattern', '/')
                    location.setdefault('match', 'exact')
                    location.setdefault('type', 'static')
                    location.setdefault('params', {})
                    location.setdefault('custom_conf', None)
                    location.setdefault('custom_conf_override', False)
                    location.setdefault('path', ws['root'])
                    location.setdefault('path_append_pattern', False)

                    if location['type'] == 'static':
                        location['params'].setdefault('autoindex', False)

                    if location['type'] == 'proxy':
                        location['params'].setdefault('url', 'http://127.0.0.1')

                    if location['type'] == 'fcgi':
                        location['params'].setdefault('url', 'http://127.0.0.1:9000')

                    if location['type'] == 'app':
                        location['params'].setdefault('app', None)

        if isinstance(config, SystemConfig):
            config.data.setdefault('log_dir', '/var/log/veb')



from jadi import interface
from mako.template import Template as MakoTemplate

from .config import MainConfig, SystemConfig


@interface
class Template(object):
    name = None
    data = None

    @classmethod
    def by_name(cls, context, name):
        for template in cls.classes():
            if template.name == name:
                return template(context)

    def __init__(self, context):
        self.context = context
        self.template = MakoTemplate(self.data)

    def render(self, data={}):
        data.update({
            'main_config': MainConfig.get(self.context).data,
            'enabled_websites': [
                x for x in
                MainConfig.get(self.context).data['websites']
                if x['enabled']
            ],
            'system_config': SystemConfig.get(self.context).data,
        })
        return self.template.render(**data).replace('\n\n', '\n')



import os
import subprocess
from jadi import component

import vvv
from vvv.api.app import AppType
from vvv.api.check import Check, CheckFailure
from vvv.api.config import MainConfig
from vvv.api.configurable import Configurable
from vvv.api.template import Template
from vvv.api.util import ensure_directory


@component(Check)
class WSGIModuleCheck(Check):
    def get_instances(self):
        for website in MainConfig.get(self.context).data['websites']:
            if website['enabled']:
                for app in website['apps']:
                    if app['type'] == 'python-wsgi':
                        mod = app['params']['module']
                        if ':' in mod:
                            mod, var = mod.split(':')
                        else:
                            var = 'application'
                        yield [mod, var, website, app]

    def get_name(self, mod, var, website, app):
        return 'wsgi module importable in %s/%s' % (
            website['name'],
            app['name'],
        )

    def run(self, mod, var, website, app):
        try:
            subprocess.check_output([
                'python', '-c',
                'import %s as wsgi; print wsgi.%s' % (
                    mod, var
                )
            ], cwd=app['path'])
            return True
        except:
            raise CheckFailure('could not import %s from module %s' % (mod, var))


@component(Template)
class ConfigFileTemplate(Template):
    name = 'gunicorn'
    data = """
import multiprocessing

bind = 'unix:/var/run/veb-app-${website['name']}-${app['name']}.sock'
user = '${app['params']['user']}'
chdir = '${app['path']}'
workers = ${app['params']['workers']} or (multiprocessing.cpu_count() * 2 + 1)
accesslog = '${system_config['log_dir']}/${website['name']}/${app['name']}.access.log'
errorlog =  '${system_config['log_dir']}/${website['name']}/${app['name']}.error.log'

${app['params']['custom_conf'] or ''}
    """


def _get_module_name(website, app):
    module_name = '%s-%s' % (website['name'], app['name'])
    return module_name.replace('-', '_').replace(' ', '_')


def _get_config_path(website, app):
    config_dir = os.path.join(vvv.config_dir, 'gunicorn')
    ensure_directory(config_dir)
    return os.path.join(config_dir, _get_module_name(website, app) + '.py')


@component(AppType)
class PythonAppType(AppType):
    name = 'python-wsgi'

    def get_access_type(self, website_config, app_config):
        return 'proxy'

    def get_access_params(self, website_config, app_config):
        return {
            'url': 'http://unix:/var/run/veb-app-%s-%s.sock' % (website_config['name'], app_config['name'])
        }

    def get_process(self, website_config, app_config):
        p = {
            'command': 'gunicorn -c %s %s' % (
                _get_config_path(website_config, app_config),
                app_config['params']['module'],
            ),
            'user': app_config['params']['user'],
            'environment': app_config['params']['environment'],
            'directory': app_config['path'],
        }
        virtualenv = app_config['params']['virtualenv']
        if virtualenv:
            p['environment'] = 'PATH="%s:%s"' % (
                os.path.join(virtualenv, 'bin'),
                os.environ['PATH'],
            )
            p['command'] = os.path.join(virtualenv, 'bin') + '/' + p['command']
        return p


@component(Configurable)
class GUnicorn(Configurable):
    name = 'gunicorn'

    def configure(self):
        for website in MainConfig.get(self.context).data['websites']:
            if website['enabled']:
                for app in website['apps']:
                    if app['type'] == 'python-wsgi':
                        open(_get_config_path(website, app), 'w').write(
                            Template.by_name(self.context, 'gunicorn').render(data={
                                'website': website,
                                'app': app,
                            })
                        )



import os
import yaml
from jadi import service

import vvv
from vvv.api.plugin import Plugin


class ConfigFile(object):
    subpath = None

    def __init__(self, context):
        self.context = context
        self.path = os.path.join(vvv.config_dir, self.subpath)
        self.load()

    def load(self):
        if os.path.exists(self.path):
            self.data = yaml.load(open(self.path))
        else:
            self.data = {}

        for plugin in Plugin.all(self.context):
            plugin.add_config_defaults(self)

    def save(self):
        with open(self.path, 'w') as f:
            f.write(yaml.safe_dump(
                self.data,
                default_flow_style=False,
                encoding='utf-8',
                allow_unicode=True
            ))


@service
class MainConfig(ConfigFile):
    subpath = 'config.yml'


@service
class SystemConfig(ConfigFile):
    subpath = 'system.yml'



#!/usr/bin/env python
from distutils.core import setup
from setuptools import find_packages


__requires = filter(None, open('requirements.txt').read().splitlines())

setup(
    name='vvveb',
    version='1.0.4',
    install_requires=__requires,
    description='An automatic website hosting configurator',
    author='Eugene Pankov',
    author_email='e@ajenti.org',
    url='https://github.com/ajenti/veb',
    scripts=['veb'],
    packages=find_packages(),
)



from jadi import component

from vvv.api.app import AppType


@component(AppType)
class NodeJSType(AppType):
    name = 'nodejs'

    def get_access_type(self, website_config, app_config):
        return 'proxy'

    def get_access_params(self, website_config, app_config):
        return {
            'url': 'http://127.0.0.1:%s' % app_config['params']['port']
        }

    def get_process(self, website_config, app_config):
        p = {
            'command': '%s %s' % (
                app_config['params']['node_binary'],
                app_config['params']['script'] or '.',
            ),
            'user': app_config['params']['user'],
            'environment': app_config['params']['environment'],
            'directory': app_config['path'],
        }
        return p
